@@ -24,6 +24,7 @@ SECURE_SSLCONTEXT.check_hostname = True
 
 INSECURE_SSLCONTEXT = ssl.SSLContext(protocol=ssl.PROTOCOL_TLS_CLIENT)
 INSECURE_SSLCONTEXT.check_hostname = False
+INSECURE_SSLCONTEXT.verify_mode = ssl.CERT_NONE
 
 
 class IncomingContent(Content):
@@ -80,7 +81,6 @@ class UpgradeResponse(Exception):
 
 
 class ClientConnection(asyncio.Protocol):
-
     __slots__ = (
         "loop",
         "pool",
@@ -271,11 +271,15 @@ class ClientConnection(asyncio.Protocol):
         # NB: check if headers declare a content-length
         if self._has_content():
             self.response.content = IncomingContent(
-                self.response.get_single_header(b"content-type")
+                (
+                    self.response.get_first_header(b"content-type")
+                    or b"application/octet-stream"
+                )
             )
         self.response_ready.set()
 
     def _has_content(self) -> bool:
+        assert self.response is not None
         content_length = self.response.get_first_header(b"content-length")
 
         if content_length:
